<?php declare(strict_types=1);

namespace Lewisdale\Webmentions\Gateways;

use Lewisdale\Webmentions\Models\Author;
use Lewisdale\Webmentions\Models\MentionType;
use Lewisdale\Webmentions\Models\Webmention;
use PDO;

class SqliteGateway extends WebmentionGatewayInterface
{
    private readonly PDO $connection;

    function __construct(string $name)
    {
        $this->connection = new PDO("sqlite:$name");
        $this->up();
    }

    protected function up(): void
    {
        // Create Webmention table
        $sql = <<<SQL
        PRAGMA foreign_keys = ON;
        CREATE TABLE IF NOT EXISTS authors (
            id INTEGER PRIMARY KEY,
            name TEXT NOT NULL,
            url TEXT,
            photo TEXT,
            UNIQUE(name, url)
        );
        CREATE TABLE IF NOT EXISTS webmentions (
            id INTEGER PRIMARY KEY,
            target TEXT NOT NULL,
            source TEXT NOT NULL,
            type TEXT NOT NULL,
            content TEXT,
            author INTEGER,
            FOREIGN KEY(author) REFERENCES authors(id)          
        );
        SQL;

        $this->connection->exec($sql);
    }

    public function get(int $id): ?Webmention
    {
        $sql = <<<SQL
        SELECT webmentions.id as id,
               webmentions.source as source,
               webmentions.target as target,
               webmentions.type as type,
               webmentions.content as content,
               authors.id as author,
               authors.name as author_name,
               authors.url as author_url,
               authors.photo as author_photo
        FROM webmentions
                 LEFT JOIN 
                    authors ON webmentions.author=authors.id
                WHERE webmentions.id=:id
                LIMIT 1;
        SQL;
        $statement = $this->connection->prepare($sql);
        $statement->execute(["id" => $id]);
        $row = $statement->fetch(PDO::FETCH_ASSOC);
        $statement->closeCursor();

        if ($row) {
            return new Webmention(
                $row["id"],
                $row["target"],
                $row["source"],
                MentionType::from($row["type"]),
                $row["content"],
                new Author((int)$row["author"], $row["author_name"], $row["author_url"], $row["author_photo"])
            );
        }

        return null;
    }

    public function getByPost(string $post): array
    {
        $sql = <<<SQL
        SELECT webmentions.id as id,
               webmentions.source as source,
               webmentions.target as target,
               webmentions.type as type,
               webmentions.content as content,
               authors.id as author,
               authors.name as author_name,
               authors.url as author_url,
               authors.photo as author_photo
        FROM webmentions
                 LEFT JOIN 
                    authors ON webmentions.author=authors.id
        WHERE target=:post;
        SQL;
        $statement = $this->connection->prepare($sql);
        $statement->execute(["post" => $post]);

        $mentions = array_map([$this, 'map_row'], $statement->fetchAll(PDO::FETCH_ASSOC));
        $statement->closeCursor();

        return $mentions;
    }

    public function save(Webmention $webmention): ?int
    {
        $author = $this->saveAuthor($webmention->author);

        $sql = <<< SQL
        REPLACE INTO webmentions
            (id, target, source, type, content, author)
        VALUES
            (:id, :target, :source, :type, :content, :author);
        SQL;

        $statement = $this->connection->prepare($sql);
        $success = $statement->execute([
            "id" => $webmention->id,
            "target" => $webmention->target,
            "source" => $webmention->source,
            "type" => $webmention->type->toString(),
            "content" => $webmention->content,
            "author" => $author,
        ]);

        $statement->closeCursor();

        return $success ? (int)$this->connection->lastInsertId() : null;
    }

    protected function saveAuthor(Author $author): ?int
    {
        if (!$author->name || !$author->url) {
            return null;
        }

        $sql = <<< SQL
        INSERT OR IGNORE INTO authors
            (id, name, url, photo)
        VALUES
            (:id, :name, :url, :photo);
        SQL;

        $statement = $this->connection->prepare($sql);
        $success = $statement->execute([
            "id" => $author->id,
            "name" => $author->name,
            "url" => $author->url,
            "photo" => $author->photo,
        ]);
        $statement->closeCursor();

        return $this->findAuthor($author->name, $author->url)?->id;
    }

    protected function findAuthor(string $name, string $url): ?Author
    {

        $sql = "SELECT id, name, url, photo FROM authors WHERE url=:url AND name=:name";
        $statement = $this->connection->prepare($sql);

        $success = $statement->execute([
            "url" => $url,
            "name" => $name,
        ]);

        if (!$success) {
            return null;
        }

        $row = $statement->fetch(PDO::FETCH_ASSOC);
        $statement->closeCursor();

        return $row ? new Author(
            (int)$row['id'],
            $row['name'],
            $row['url'],
            $row['photo']
        ) : null;
    }

    public function delete(Webmention $webmention): void
    {
        $sql = "DELETE FROM webmentions WHERE id=:id;";
        $statement = $this->connection->prepare($sql);
        $statement->execute(["id" => $webmention->id]);
        $statement->closeCursor();
    }

    public function find(array $values): array
    {
        $keys = implode(" AND ", array_map(function ($v) {
            return "$v=:$v";
        }, array_keys($values)));
        $sql = <<<SQL
        SELECT webmentions.id as id,
               webmentions.source as source,
               webmentions.target as target,
               webmentions.type as type,
               webmentions.content as content,
               authors.id as author,
               authors.name as author_name,
               authors.url as author_url,
               authors.photo as author_photo
        FROM webmentions
             LEFT JOIN 
                authors ON webmentions.author=authors.id
        WHERE {$keys}
        SQL;

        $statement = $this->connection->prepare($sql);
        $statement->execute($values);

        return array_map([$this, 'map_row'], $statement->fetchAll());
    }

    public function list(): array
    {
        $sql = <<<SQL
        SELECT webmentions.id as id,
               webmentions.source as source,
               webmentions.target as target,
               webmentions.type as type,
               webmentions.content as content,
               authors.id as author,
               authors.name as author_name,
               authors.url as author_url,
               authors.photo as author_photo
        FROM webmentions
             LEFT JOIN 
                authors ON webmentions.author=authors.id
        SQL;

        $statement = $this->connection->query($sql);
        $mentions = array_map([$this, 'map_row'], $statement->fetchAll(PDO::FETCH_ASSOC));
        $statement->closeCursor();
        return $mentions;
    }

    private function map_row(array|false $row): ?Webmention
    {
        if ($row) {
            return new Webmention(
                $row["id"],
                $row["target"],
                $row["source"],
                MentionType::from($row["type"]),
                $row["content"],
                new Author(
                    (int)$row["author"],
                    $row["author_name"],
                    $row["author_url"],
                    $row["author_photo"]
                )
            );
        }
        return null;
    }
}

?>